package com.lonestranger.onescreen;

import it.marteEngine.ResourceManager;
import it.marteEngine.SFX;
import it.marteEngine.World;
import it.marteEngine.entity.Entity;
import it.marteEngine.entity.TextEntity;

import org.newdawn.slick.Color;
import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.util.Log;

import com.lonestranger.onescreen.entity.Background;
import com.lonestranger.onescreen.entity.Blocker;
import com.lonestranger.onescreen.entity.PongScreen;
import com.lonestranger.onescreen.entity.ScreenEntity;

public class GameWorld extends World {

	public static int STATE_NONE 	= 0;
	public static int STATE_INIT 	= 10;
	public static int STATE_START 	= 20;
	public static int STATE_POP 	= 25;
	public static int STATE_PLAY 	= 30;
	public static int STATE_DONE 	= 40;
	public static int STATE_END		= 50;
	
	protected int STATE = STATE_NONE;
	
	
	protected ScreenEntity screenEntity;
	
	Rectangle window;
	Rectangle screenTargetArea;
	Rectangle lowerBlockTargetArea;

	int score = 0;
	int minBlockers = 3;
	int lrmargin = 100;
	
	int nerves = 0;
	
	TextEntity scoreEntity;
	TextEntity rulesEntity;
	TextEntity bylineEntity;
	TextEntity nervesEntity;
	TextEntity finishedEntity;
	PongScreen pong;
	
	String finishedText = "They really got to your nerves! ";
	
	int mouseX = 0;
	int mouseY = 0;
	
	public GameWorld(int id) {
		super(id);
	}

	@Override
	public void init(GameContainer container, StateBasedGame game)
			throws SlickException {

		super.init(container, game);

		STATE = STATE_INIT;
		// create screen
		screenEntity = new ScreenEntity(container.getWidth()/2 - (320/2), container.getHeight()/2 - (240/2), 320, 240);
		add(screenEntity);
		
		
		// set up rectangles
		window = new Rectangle(0,0,container.getWidth(), container.getHeight());
		screenTargetArea = new Rectangle(screenEntity.x, screenEntity.y, screenEntity.width, screenEntity.height);
		lowerBlockTargetArea = new Rectangle(screenEntity.x - lrmargin, screenEntity.y, screenEntity.width + lrmargin, screenEntity.height );
		
		//Blocker block = createNewBlocker(Blocker.BLOCKER_SNOWMAN);
		//add(block);
		
		Font f = container.getDefaultFont();
		scoreEntity = new TextEntity(10, 40, f, "0");
		scoreEntity.x = (width / 2) - (scoreEntity.width /2);
		add(scoreEntity);
		
		rulesEntity = new TextEntity(0, 20, f, "Press W/S to move paddle, click with mouse to dismiss distractions.");
		rulesEntity.x = (width / 2) - (rulesEntity.width / 2);
		add(rulesEntity);
		
		bylineEntity = new TextEntity(0, 0, f, "You're Blocking The Screen! by LoneStranger for Ludum Dare 31");
		bylineEntity.x = (width / 2) - (bylineEntity.width / 2);
		add(bylineEntity);
		
		nervesEntity = new TextEntity(0,60, f, "---------");
		nervesEntity.x = (width / 2) - (nervesEntity.width / 2);
		add(nervesEntity);
		
		finishedEntity = new TextEntity(0, 120, f, finishedText);
		finishedEntity.setColor(Color.red);
		finishedEntity.x = (width / 2) - (finishedEntity.width / 2);
		finishedEntity.visible = false;
		add(finishedEntity);		
		
		
		// 240, 180, 320,20
		pong = new PongScreen(260, 232, 272,152, this);
		add(pong);
		
		Background b = new Background(0,0);
		b.resize(container.getWidth(), container.getHeight());
		add(b);

//		Camera c = new Camera(this, player, container.getWidth(), container.getHeight(), 512, 512, new Vector2f(32,32));		
//		setCamera(c);
//		setCameraOn(player);
		
		// set initial nerves
		modNerves(100);
		
		SFX.setSoundVolume(.5f);
		STATE = STATE_START;
		
	}

	@Override
	public void update(GameContainer container, StateBasedGame game, int delta)
			throws SlickException {
		// TODO Auto-generated method stub
		super.update(container, game, delta);
		
		mouseX = container.getInput().getMouseX();
		mouseY = container.getInput().getMouseY();
				
		boolean click = container.getInput().isMousePressed(Input.MOUSE_LEFT_BUTTON);		

		if (STATE == STATE_START)
		{
			STATE = STATE_PLAY;
		}
		if (STATE == STATE_POP)
		{
			int r = (int)(Math.random() * 100);
			if (r < 3)
				add(createNewBlocker(Blocker.BLOCKER_SNOWMAN));
			else if (r < 50)
				add(createNewBlocker(Blocker.BLOCKER_AIRPLANE));
			else if (r < 75)
				add(createNewBlocker(Blocker.BLOCKER_ADELAIDE));
			else if (r < 100)
				add(createNewBlocker(Blocker.BLOCKER_HUNTER));
			
			
			STATE = STATE_PLAY;
		}
		if (STATE == STATE_PLAY)
		{
			if (this.getNrOfEntities("blocker") < minBlockers)
				STATE = STATE_POP;
			
			
			scoreEntity.x = (width / 2) - (scoreEntity.width / 2);
			
			nervesEntity.x = (width / 2) - (nervesEntity.width / 2);
			
			if (nerves <= 0)
				STATE = STATE_DONE;
			
		}
		
		if (STATE == STATE_DONE)
		{
			pong.setPaused(true);
			finishedEntity.setColor(Color.red);
			finishedEntity.setText(finishedText + " You scored " + score + " points.");
			finishedEntity.x = (width / 2) - (finishedEntity.width / 2);
			finishedEntity.visible = true;
			STATE = STATE_END;
		}
		
		
		
	}
			
	
	private Blocker createNewBlocker(int blockerType) throws SlickException
	{
		// Full Left Side
		//sourceArea = new Rectangle(0 - blimage.getWidth(), 0, blimage.getWidth(), container.getHeight());
		// Full Right Side
		//sourceArea = new Rectangle(container.getWidth(), 0, blimage.getWidth(), container.getHeight());
		// Full Bottom
		//sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight(), container.getWidth(), 1);
		// Full Top
		//sourceArea = new Rectangle(0 - blimage.getWidth(), 0 - blimage.getHeight(), container.getWidth(), 1);		
		// Lower Left Side
		//sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
		// Lower Right Side
		//sourceArea = new Rectangle(container.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
		
		
		Blocker block = null;
		double loc;
		Rectangle sourceArea = null;
		
		if (blockerType == Blocker.BLOCKER_SQUARE)
		{
			
			Image blimage = ResourceManager.getImage("blockersquare");
			
			int r = (int) (Math.random() * 3);
			if (r == 0)
				sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight(), container.getWidth(), 0);
			if (r == 1)
				sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
			if (r == 2)
				sourceArea = new Rectangle(container.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
			
			block = new Blocker(window, lowerBlockTargetArea, sourceArea, blimage, Blocker.LOC_BOTTOM);
			block.setHitBox(0,0,blimage.getWidth(), blimage.getHeight());
			block.name = "blockerSQUARE";
		}
		
		if (blockerType == Blocker.BLOCKER_HUNTER)
		{
			
			Image blimage = ResourceManager.getImage("blockerhunter");
			
			int r = (int) (Math.random() * 3);
			if (r == 0)
				sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight(), container.getWidth(), 0);
			if (r == 1)
				sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
			if (r == 2)
				sourceArea = new Rectangle(container.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
			
			block = new Blocker(window, lowerBlockTargetArea, sourceArea, blimage, Blocker.LOC_BOTTOM);
			block.setHitBox(0,0,blimage.getWidth(), blimage.getHeight());	
			block.name = "blockerHUNTER";
			
			playRandomSound("hunter",3);
			//SFX.playSound("hunter1");
			
		}		
	
		if (blockerType == Blocker.BLOCKER_ADELAIDE)
		{
			
			Image blimage = ResourceManager.getImage("blockeradelaide");
			
			int r = (int) (Math.random() * 3);
			if (r == 0)
				sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight(), container.getWidth(), 0);
			if (r == 1)
				sourceArea = new Rectangle(0 - blimage.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
			if (r == 2)
				sourceArea = new Rectangle(container.getWidth(), container.getHeight()/2, blimage.getWidth(), container.getHeight()/2);
			
			block = new Blocker(window, lowerBlockTargetArea, sourceArea, blimage, Blocker.LOC_BOTTOM);
			block.setHitBox(0,0,blimage.getWidth(), blimage.getHeight());	
			block.name = "blockerADELAIDE";
			
			playRandomSound("adelaide",3);
			
			
		}		
		if (blockerType == Blocker.BLOCKER_AIRPLANE)
		{
			
			Image blimage = ResourceManager.getImage("blockerairplane");
			
			Rectangle targetArea = null;
			
			int r = (int) (Math.random() * 2);
			if (r == 0) // left to right
			{
				blimage = blimage.getFlippedCopy(true,  false);
				sourceArea = new Rectangle(0 - blimage.getWidth(), 0, blimage.getWidth(), container.getHeight());
				targetArea = new Rectangle(container.getWidth(), 0, blimage.getWidth(), container.getHeight());
				block = new Blocker(window, targetArea, sourceArea, blimage, Blocker.LOC_BOTTOM);
				block.setHitBox(20,16,150,85);			
				block.destroyAtDestination(true);
			}
	
			if (r == 1) // right to left
			{
				sourceArea = new Rectangle(container.getWidth(), 0, blimage.getWidth(), container.getHeight());
				targetArea = new Rectangle(0 - blimage.getWidth(), 0, blimage.getWidth(), container.getHeight());
				block = new Blocker(window, targetArea, sourceArea, blimage, Blocker.LOC_BOTTOM);
				block.setHitBox(30,16,150,85);	
				block.destroyAtDestination(true);				
			}

			block.name = "blockerAIRPLANE";

			
			playRandomSound("plane",1);
			
		}		
				
		if (blockerType == Blocker.BLOCKER_SNOWMAN)
		{
			
			Image blimage = ResourceManager.getImage("blockersnowman");
			
			sourceArea = new Rectangle(0 - blimage.getWidth(), 0 - blimage.getHeight(), container.getWidth() + blimage.getWidth(), 1);
			
			block = new Blocker(window, lowerBlockTargetArea, sourceArea, blimage, Blocker.LOC_BOTTOM);
			block.setHitBox(64,0,214,350);	
			block.name = "blockerSNOWMAN";
			
		}
		return block;
	}
	
	
	@Override
	public void render(GameContainer container, StateBasedGame game, Graphics g)
			throws SlickException {
		super.render(container, game, g);

		g.setColor(Color.white);
		g.fillRect(mouseX - 3, mouseY - 3, 7, 7);
		g.setColor(Color.red);
		g.fillRect(mouseX - 1, mouseY - 1, 3, 3);
		g.setColor(Color.black);
		g.fillRect(mouseX - 0, mouseY - 0, 1, 1);
		
		
		
	}



	@Override
	public void mouseClicked(int button, int x, int y, int clickCount) {
		// TODO Auto-generated method stub
		super.mouseClicked(button, x, y, clickCount);
		
		Log.info("Checking Mouse(" + x + "," + y + ")");
		for (Entity e : getEntities())
		{
			//System.out.println("entity: " + e);
			if ((e.name != null) && e.name.startsWith("blocker"))
			{
				//Log.info("   checking - " + e.toString());
				if (e.collidePoint(x,  y))
				{
					//Log.info("        Hit - " + e.toString());
					((Blocker)e).goAway();
					modScore(10);
					modNerves(1);
					
				}
			}
		}
		
		
	}
	
	public void modScore(int m)
	{
		score += m;
		scoreEntity.setText("" + score);
	}

	public void modNerves(int m)
	{
		StringBuffer ntext = new StringBuffer("[");
		nerves += m;
	
		if (nerves > 100)
			nerves = 100;
		
		for (int i = 0; i < (nerves/10); i++)
			ntext.append("-");
		ntext.append("]");
		nervesEntity.setText("" + ntext);
	}
	
	public void playRandomSound(String s, int n)
	{
		int r = (int)(Math.random() * n);
		SFX.playSound(s + (r+1));
		//System.out.println("playing: " + s + (r+1));
	}
	
	

}