package com.lonestranger.onescreen.entity;

import java.util.Random;

import org.lwjgl.input.Controller;
import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Point;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Vector2f;

import com.lonestranger.onescreen.GameWorld;

import it.marteEngine.ResourceManager;
import it.marteEngine.entity.Entity;
import it.marteEngine.entity.PhysicsEntity;
import it.marteEngine.tween.Ease;
import it.marteEngine.tween.LinearMotion;
import it.marteEngine.tween.Tween.TweenerMode;

public class Blocker extends PhysicsEntity {

	protected Vector2f targetLocation;
	
	public static int LOC_BOTTOM 	= 0;
	public static int LOC_LEFT		= 1;
	public static int LOC_RIGHT 	= 2;
	public static int LOC_TOP 		= 3;

	public static int BLOCKER_SQUARE = 0;
	public static int BLOCKER_HUNTER = 1;
	public static int BLOCKER_AIRPLANE = 2;
	public static int BLOCKER_SNOWMAN = 3;
	public static int BLOCKER_TIGERTWO = 4;
	public static int BLOCKER_BABYKITTY = 5;
	public static int BLOCKER_ADELAIDE = 6;
	
	public static int STATE_NONE 	= 0;
	public static int STATE_INIT 	= 5;
	public static int STATE_ENTER 	= 10;
	public static int STATE_WAIT 	= 20;
	public static int STATE_PICKEXIT= 30;
	public static int STATE_EXIT 	= 31;
	public static int STATE_DESTROY = 32;
	
	
	protected int STATE;
	
	private float mouseX = 0;
	private float mouseY = 0;
	
	private boolean dismissed = false;
	
	public int speed = 1;
	protected float waitTime = 0; 
	
	protected Rectangle windowSize;
/*	public Rectangle STARTAREA_BOTTOM = new Rectangle(0,0,0,0);
	public Rectangle STARTAREA_LEFT = new Rectangle(0,0,0,0);
	public Rectangle STARTAREA_RIGHT = new Rectangle(0,0,0,0);
	public Rectangle STARTAREA_TOP = new Rectangle(0,0,0,0);
*/
	public Rectangle targetArea = new Rectangle(0,0,0,0);
	public Rectangle sourceArea = new Rectangle(0,0,0,0);
	
	public int blockerType;
	protected int exitSide;
	protected LinearMotion move;
	
	protected boolean destroyAtDestination = false;
	
	public Blocker(Rectangle windowSize, Rectangle targetArea, Rectangle sourceArea, Image img, int exitSide) throws SlickException {
		super(0, 0);
		
		
		name = "blocker";
		this.addType("blocker");
		depth = 10;
		STATE = Blocker.STATE_INIT;
		init();

		this.windowSize = windowSize;
		//this.blockerType = blockerType;
		this.targetArea = targetArea;
		this.sourceArea = sourceArea;
		this.exitSide = exitSide;
		
		setParams(img, sourceArea);
		pickTarget();
		
		//this.setHitBox(0,0,currentImage.getWidth(), currentImage.getHeight());
		
		STATE = Blocker.STATE_ENTER;
	}

/*	public Blocker(float x, float y, String refImage) throws SlickException
	{		
		super(x, y);
		currentImage = new Image(refImage);
		init();
	}
*/
	
	private void init()
	{
		
	}
	
	
	@Override
	public void update(GameContainer container, int delta)
			throws SlickException {
		//super.update(container, delta);
		mouseX = container.getInput().getMouseX();
		mouseY = container.getInput().getMouseY();
				
		boolean click = container.getInput().isMousePressed(Input.MOUSE_LEFT_BUTTON);
		
		/*if (STATE == Blocker.STATE_ENTER || STATE == Blocker.STATE_PICKEXIT || STATE == Blocker.STATE_WAIT)
		{
			if (click)
			{
				System.out.println("testing: " + mouseX + "," + mouseY + "-" + this);
				if (collidePoint(mouseX, mouseY))
				{
					STATE = Blocker.STATE_PICKEXIT;
				
					((GameWorld)world).modScore(100);
					
				}
			}
		}
		*/
		
		//System.out.println("state=" + STATE);
		if ( (STATE == Blocker.STATE_ENTER) || (STATE == Blocker.STATE_EXIT) )
		{
			move.update(delta);
			
			x = move.getX();
			y = move.getY();
			
			if (move.isFinished() && (STATE == Blocker.STATE_ENTER) )
			{		
				if (destroyAtDestination)
					STATE = STATE_DESTROY;
				else
				{
					waitTime = 10000;
					STATE = Blocker.STATE_WAIT;
				}
			}	
			
			if(move.isFinished() && (STATE == Blocker.STATE_EXIT))
			{
				STATE = STATE_DESTROY;
			}
			
		}
		
		
		if (STATE == Blocker.STATE_WAIT)
		{
			waitTime -= delta;
			
			if (waitTime <= 0)
			{
				dismissed = false;
				STATE = Blocker.STATE_PICKEXIT;
			}
		}
		if (STATE == Blocker.STATE_PICKEXIT)
		{				
			float toX = 0; 
			float toY = 0;
			
			if (exitSide == LOC_TOP)
			{
				toX = x;
				toY = 0;
			}
			if (exitSide == LOC_BOTTOM)
			{
				toX = x;
				toY = container.getHeight();
			}
			
			if (exitSide == LOC_LEFT)
			{
				toX = 0 - currentImage.getWidth() - 100;
				toY = y;
			}

			if (exitSide == LOC_RIGHT)
			{
				toX = container.getWidth();// - currentImage.getWidth();
				toY = y;
			}
			
			if (!dismissed)
				((GameWorld)world).modNerves(-10);
			
			
			
			move = new LinearMotion(x, y, toX, toY, speed, TweenerMode.PERSIST, 100, Ease.NONE);
			STATE = Blocker.STATE_EXIT;
		}
		if (STATE == STATE_DESTROY)
		{
			STATE = STATE_NONE;
			this.destroy();
		}
		//System.out.println(this + " - " + STATE);
		
	}
	
	public void collisionResponse(Entity other) {
	
		System.out.println("Hit: " + this);
	}

	public void goAway()
	{
		dismissed = true;
		//if (STATE != Blocker.STATE_EXIT)
			STATE = Blocker.STATE_PICKEXIT;
	}
	
	public void destroyAtDestination(boolean b)
	{
		destroyAtDestination = b;
	}
	
	
	@Override
	public void render(GameContainer container, Graphics g)
			throws SlickException {
		// TODO Auto-generated method stub
		super.render(container, g);
	//	g.setColor(Color.magenta);
	//	g.fillRect(targetLocation.x, targetLocation.y, 2, 2);

	//	g.setColor(Color.yellow);
	//	g.drawRect(targetArea.getX(), targetArea.getY(), targetArea.getWidth(), targetArea.getHeight());
		
	//	g.setColor(Color.cyan);
	//	g.drawRect(x + hitboxOffsetX, y + hitboxOffsetY, hitboxWidth, hitboxHeight);
		
	//	g.setColor(Color.pink);
	//	g.fillRect(mouseX, mouseY, 3, 3);
		
		
	}

	public void setWindowSize(float x, float y, float w, float h)
	{
		windowSize = null;
		windowSize = new Rectangle(x,y,w,h);
		
	}
	
	public void setParams(Image img, Rectangle sourceArea) throws SlickException
	{
		currentImage = img;
		width = currentImage.getWidth();
		height = currentImage.getHeight();
		
		System.out.println("setParams: " + width + "x" + height);
		
		x = (float) (sourceArea.getX() + (Math.random() * sourceArea.getWidth()));
		y = (float) (sourceArea.getY() + (Math.random() * sourceArea.getHeight()));
				
	}
	
	public void pickTarget()
	{
		
		int sx = (int) (Math.random() * targetArea.getWidth());
		int sy = (int) (Math.random() * targetArea.getHeight());
		
		targetLocation = new Vector2f();
		targetLocation.x = sx + targetArea.getX();
		targetLocation.y = sy + targetArea.getY();
		
		//double theta = 360 * Math.random();
//		double theta = getAngleToPosition(targetLocation);

//		Point direction = new Point(0,0);
//		direction.setX((float)Math.cos(theta));
//		direction.setY((float)Math.sin(theta));
		
		//acceleration.x = direction.getX() * speed;
		//acceleration.y = direction.getY() * speed;		
		
		
		move = new LinearMotion(x, y, targetLocation.x, targetLocation.y, speed, TweenerMode.ONESHOT, 200, Ease.NONE);//, gravity, sy, active);
		
		
	}
	
	
	
}
