package com.lonestranger.onescreen.entity;


import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Rectangle;

import com.lonestranger.onescreen.GameWorld;

import it.marteEngine.SFX;
import it.marteEngine.entity.Entity;

public class PongScreen extends Entity {

	public int STATE_NONE = 0;
	public int STATE_START = 1;
	public int STATE_PLAY = 2;
	public int STATE_RESET = 3;
	public int STATE_END = 4;
	
	public int STATE = STATE_NONE;
	
	int ballX;
	int ballY;
	
	int paddleLeftX;
	int paddleLeftY;
	
	int paddleRightX;
	int paddleRightY;
	
	int paddleHeight = 30;
	int paddleWidth = 4;
	
	int ballHeight = 4;
	int ballWidth = 4;
	float ballDX = 0;
	float ballDY = 0;
	
	int paddleSpeed = 6;
	float ballSpeed = 2f;
	float maxBallSpeed = 4;
	
	int scoreLeft = 0;
	int scoreRight = 0;
	
	Rectangle rectPaddleLeft;
	Rectangle rectPaddleRight;
	Rectangle rectBall;
	
	int multiplier = 1;
	
	Color backgroundColor = Color.black;
	
	boolean compLeft = false;
	boolean compRight = true;
	
	GameWorld gworld;
	
	int waitTimer = 0;
	int timeToWait = 3000;
	int timeToWaitFirst = 10000;
	
	private boolean paused = false;
	
	public PongScreen(float x, float y, int width, int height, GameWorld world) {
		super(x, y);

		gworld = world;
		
		this.width = width;
		this.height = height;
		
		depth = 0;
	
		define("LEFTUP", Input.KEY_W);
		define("LEFTDOWN", Input.KEY_S);
		
		define("RIGHTUP", Input.KEY_UP);
		define("RIGHTDOWN", Input.KEY_DOWN);
	
		paddleLeftX = 10;
		paddleRightX = width - 10;
		
		paddleLeftY = (height / 2) - (paddleHeight/2);
		paddleRightY = (height / 2) - (paddleHeight/2);
		
		resetBall();
		pickBallDir();
	
		rectPaddleLeft 	= new Rectangle(paddleLeftX, paddleLeftY, paddleWidth, paddleHeight);
		rectPaddleRight = new Rectangle(paddleRightX, paddleRightY, paddleWidth, paddleHeight);
		rectBall = new Rectangle(ballX, ballY, ballWidth, ballHeight);
		
		waitTimer = timeToWaitFirst;
		STATE = STATE_START;
	}


	@Override
	public void update(GameContainer container, int delta)
			throws SlickException {
		// TODO Auto-generated method stub
		super.update(container, delta);
		
		if (!paused)
		{
			
			if (STATE == STATE_START)
			{
				waitTimer -= delta;
				if (waitTimer <= 0)
					STATE = STATE_PLAY;
			}
			
			if (STATE == STATE_PLAY || STATE == STATE_START)
			{
				
				if (!compLeft)
				{
					if (check("LEFTUP"))
						paddleLeftY -= paddleSpeed;
					if (check("LEFTDOWN"))
						paddleLeftY += paddleSpeed;
				}
				else
				{
					if (ballX < (paddleLeftY + (paddleHeight/2)))
						paddleLeftY -= paddleSpeed;			
					if (ballX > (paddleLeftY + (paddleHeight/2)))
						paddleLeftY += paddleSpeed;			
		
				}
		
				if (!compRight)
				{
					if (check("RIGHTUP"))
						paddleRightY -= paddleSpeed;
					if (check("RIGHTDOWN"))
						paddleRightY += paddleSpeed;
				}
				else
				{
					if (ballY < (paddleRightY + (paddleHeight/2)))
						paddleRightY -= paddleSpeed;			
					if (ballY > (paddleRightY + (paddleHeight/2)))
						paddleRightY += paddleSpeed;			
		
				}		
				
				
				if (paddleLeftY < 0)
					paddleLeftY = 0;
				if (paddleLeftY > (height - paddleHeight))
					paddleLeftY = height - paddleHeight;
				
				
				if (paddleRightY < 0)
					paddleRightY = 0;
				if (paddleRightY > (height - paddleHeight))
					paddleRightY = height - paddleHeight;
			}
			
			if (STATE == STATE_PLAY){
					
				
				ballX += ballDX * ballSpeed;
				ballY += ballDY * ballSpeed;
				
				// off left or right?
				if (ballX < (0 - ballWidth/2))
				{
					SFX.playSound("ballzap");
					STATE = STATE_RESET;
					//ballDX = -ballDX;
				}
				if (ballX > (width + ballWidth/2))
				{
					SFX.playSound("ballzap");
	
					STATE = STATE_RESET;
					//ballDX = -ballDX;
				}
				
				// off top or bottom? reflect.
				if (ballY < (0 - ballHeight/2))
					ballDY = -ballDY;
				if (ballY > (height + ballHeight/2))
					ballDY = -ballDY;
			}
			
			if (STATE == STATE_RESET)
			{
				resetBall();
				pickBallDir();
				waitTimer = timeToWait;
				STATE = STATE_START;
			}
			
			rectPaddleLeft.setX(paddleLeftX);
			rectPaddleLeft.setY(paddleLeftY);
			rectPaddleRight.setX(paddleRightX);
			rectPaddleRight.setY(paddleRightY);
			rectBall.setX(ballX);
			rectBall.setY(ballY);
			
			if (STATE == STATE_PLAY)
			{	
				if (rectPaddleLeft.intersects(rectBall))
				{
					ballDX = -ballDX;
					ballX += ((paddleLeftX + paddleWidth) - ballX);
					
					SFX.playSound("ballthunk");
	
					ballSpeed += .05;
					if (ballSpeed > maxBallSpeed)
						ballSpeed = maxBallSpeed;					
					scoreLeft += 10;					
					if (!compLeft)
					{
						gworld.modNerves(1);
						gworld.modScore(100);
					}
				}
				if (rectPaddleRight.intersects(rectBall))
				{
					ballDX = -ballDX;
					ballX -= ((ballX + ballWidth) - paddleRightX);
	
					SFX.playSound("ballthunk");

					ballSpeed += .05;
					if (ballSpeed > maxBallSpeed)
						ballSpeed = maxBallSpeed;
					scoreRight += 10;
					if (!compRight)
					{
						gworld.modNerves(1);
						gworld.modScore(100);
					}
				}
				//System.out.println(ballSpeed);
			}
		}
	}


	@Override
	public void render(GameContainer container, Graphics g)
			throws SlickException {
		// TODO Auto-generated method stub
		super.render(container, g);

		g.setColor(backgroundColor);
		g.fillRect(x, y, width, height);

		// draw paddles
		g.setColor(Color.white);
		g.fillRect(x + paddleLeftX, y + paddleLeftY, paddleWidth, paddleHeight);
		g.fillRect(x + paddleRightX, y + paddleRightY, paddleWidth, paddleHeight);
		
		g.setColor(Color.white);
		g.fillRect(x + ballX, y + ballY, ballWidth, ballHeight);
		
		// Text
		g.setColor(Color.white);
		g.drawString("PONG", x + (width / 2) - 17, y + 1);
		
		g.drawString(String.valueOf(scoreLeft), x + (width / 2) - 50, y + 1);
		g.drawString(String.valueOf(scoreRight), x + (width / 2) + 50, y + 1);
		
		
		if (waitTimer > 0)
		{
			g.drawString(String.valueOf((waitTimer + 1000)/1000), x + (width /2) - 5,  y + 25);
		}
		
	}

	private void resetBall()
	{
		ballSpeed = 2;
		ballX = width /2 - ballWidth;
		ballY = height/2 - ballHeight;		
	}

	private void pickBallDir()
	{
		ballDX = (int)(Math.random() * 2);	
		ballDY = (int)(Math.random() * 2);
		
		// fix if set to 0. Change this later when using non-integers for direction
		if (ballDX == 0)
			ballDX = -1;
		if (ballDY == 0)
			ballDY = -1;
	}
	
	public void setPaused(boolean b)
	{
		paused = b;
	}
	
}
